@@ -1,4 +1,4 @@
-/* 09apr16abu
+/* 13jun18abu
  * (c) Software Lab. Alexander Burger
  */
 
@@ -9,60 +9,65 @@ function renderCanvas(cvs, lst) {
    for (i = 0; i < lst.length; ++i) {
       switch ((cmd = lst[i])[0]) {  // Sync with "@lib/canvas.l"
       /*** Functions ***/
-      case 1:  // (csFillText Str X Y)
+      case 1:  // (csFont Str)
+         ctx.font = cmd[1];
+      case 2:  // (csFillText Str X Y)
          ctx.fillText(cmd[1], cmd[2], cmd[3]);
          break;
-      case 2:  // (csStrokeLine X1 Y1 X2 Y2)
+      case 3:  // (csStrokeLine X1 Y1 X2 Y2)
          ctx.beginPath();
          ctx.moveTo(cmd[1], cmd[2]);
          ctx.lineTo(cmd[3], cmd[4]);
          ctx.closePath();
          ctx.stroke();
          break;
-      case 3:  // (csClearRect X Y DX DY)
+      case 4:  // (csClearRect X Y DX DY)
          ctx.clearRect(cmd[1], cmd[2], cmd[3], cmd[4]);
          break;
-      case 4:  // (csStrokeRect X Y DX DY)
+      case 5:  // (csStrokeRect X Y DX DY)
          ctx.strokeRect(cmd[1], cmd[2], cmd[3], cmd[4]);
          break;
-      case 5:  // (csFillRect X Y DX DY)
+      case 6:  // (csFillRect X Y DX DY)
          ctx.fillRect(cmd[1], cmd[2], cmd[3], cmd[4]);
          break;
-      case 6:  // (csBeginPath)
+      case 7:  // (csBeginPath)
          ctx.beginPath();
          break;
-      case 7:  // (csClosePath)
+      case 8:  // (csClosePath)
          ctx.closePath();
          break;
-      case 8:  // (csMoveTo X Y)
+      case 9:  // (csMoveTo X Y)
          ctx.moveTo(cmd[1], cmd[2]);
          break;
-      case 9:  // (csLineTo X Y)
+      case 10:  // (csLineTo X Y)
          ctx.lineTo(cmd[1], cmd[2]);
          break;
-      case 10:  // (csBezierCurveTo X1 Y1 X2 Y2 X Y)
+      case 11:  // (csBezierCurveTo X1 Y1 X2 Y2 X Y)
          ctx.bezierCurveTo(cmd[1], cmd[2], cmd[3], cmd[4], cmd[5], cmd[6]);
          break;
-      case 11:  // (csLine X1 Y1 X2 Y2)
+      case 12:  // (csQuadraticCurveTo X1 Y1 X2 Y2)
+         ctx.quadraticCurveTo(cmd[1], cmd[2], cmd[3], cmd[4]);
+         break;
+      case 13:  // (csLine X1 Y1 X2 Y2)
          ctx.moveTo(cmd[1], cmd[2]);
          ctx.lineTo(cmd[3], cmd[4]);
          break;
-      case 12:  // (csRect X Y DX DY)
+      case 14:  // (csRect X Y DX DY)
          ctx.rect(cmd[1], cmd[2], cmd[3], cmd[4]);
          break;
-      case 13:  // (csArc X Y R A B F)
+      case 15:  // (csArc X Y R A B F)
          ctx.arc(cmd[1], cmd[2], cmd[3], cmd[4], cmd[5], cmd[6]);
          break;
-      case 14:  // (csStroke)
+      case 16:  // (csStroke)
          ctx.stroke();
          break;
-      case 15:  // (csFill)
+      case 17:  // (csFill)
          ctx.fill();
          break;
-      case 16:  // (csClip)
+      case 18:  // (csClip)
          ctx.clip();
          break;
-      case 17:  // (csDef Key DX DY Lst), (csDef Key)
+      case 19:  // (csDef Key DX DY Lst), (csDef Key)
          if (!cvs.pre)
             cvs.pre = new Array();
          var buf = cvs.pre[cmd[1]] = document.createElement('canvas');
@@ -77,17 +82,17 @@ function renderCanvas(cvs, lst) {
             buf.getContext("2d").drawImage(cvs, 0, 0);
          }
          break;
-      case 18:  // (csDraw Key X Y)
+      case 20:  // (csDraw Key X Y)
          var buf = cvs.pre[cmd[1]];
          ctx.clearRect(cmd[2], cmd[3], buf.width, buf.height);
          ctx.drawImage(buf, cmd[2], cmd[3]);
          break;
-      case 19:  // (csDrawDots DX DY Lst)
+      case 21:  // (csDrawDots DX DY Lst)
          if (cmd[3])
             for (j = 0; j < cmd[3].length; j += 2)
                ctx.fillRect(cmd[3][j], cmd[3][j+1], cmd[1], cmd[2]);
          break;
-      case 20:  // (csDrawImage Img X Y Lst DX DY)
+      case 22:  // (csDrawImage Img X Y Lst DX DY)
          var img = new Image();
          img.src = cmd[1];
          (function (img, cmd) {
@@ -104,50 +109,50 @@ function renderCanvas(cvs, lst) {
                   renderCanvas(cvs, cmd[4]);
             } } )(img, cmd);
          break;
-      case 21:  // (csTranslate X Y)
+      case 23:  // (csTranslate X Y)
          ctx.translate(cmd[1], cmd[2]);
          break;
-      case 22:  // (csRotate A)
+      case 24:  // (csRotate A)
          ctx.rotate(cmd[1]);
          break;
-      case 23:  // (csScale X Y)
+      case 25:  // (csScale X Y)
          ctx.scale(cmd[1], cmd[2]);
          break;
-      case 24:  // (csSave)
+      case 26:  // (csSave)
          ctx.save();
          break;
-      case 25:  // (csRestore)
+      case 27:  // (csRestore)
          ctx.restore();
          break;
       /*** Variables ***/
-      case 26:  // (csCursor Lst)
+      case 28:  // (csCursor Lst)
          cvs.curs = cmd[1];
          break;
-      case 27:  // (csFillStyle V)
+      case 29:  // (csFillStyle V)
          ctx.fillStyle = cmd[1];
          break;
-      case 28:  // (csStrokeStyle V)
+      case 30:  // (csStrokeStyle V)
          ctx.strokeStyle = cmd[1];
          break;
-      case 29:  // (csGlobalAlpha V)
+      case 31:  // (csGlobalAlpha V)
          ctx.globalAlpha = cmd[1];
          break;
-      case 30:  // (csLineWidth V)
+      case 32:  // (csLineWidth V)
          ctx.lineWidth = cmd[1];
          break;
-      case 31:  // (csLineCap V)
+      case 33:  // (csLineCap V)
          ctx.lineCap = cmd[1];
          break;
-      case 32:  // (csLineJoin V)
+      case 34:  // (csLineJoin V)
          ctx.lineJoin = cmd[1];
          break;
-      case 33:  // (csMiterLimit V)
+      case 35:  // (csMiterLimit V)
          ctx.miterLimit = cmd[1];
          break;
-      case 34:  // (csGlobalCompositeOperation V)
+      case 36:  // (csGlobalCompositeOperation V)
          ctx.globalCompositeOperation = cmd[1];
          break;
-      case 35:  // (csPost)
+      case 37:  // (csPost)
          cvs.post = true;
          break;
       }
