@@ -1,4 +1,4 @@
-/* 23may18abu
+/* 21sep18abu
  * (c) Software Lab. Alexander Burger
  */
 
@@ -94,6 +94,14 @@ function idFocus(fld) {
    setTimeout(function() {document.getElementById(fld).focus()}, 420);
 }
 
+function setCheck(fld, val) {
+   var lst = document.getElementsByName(fld.name);
+
+   for (var i = 1;  i < lst.length; ++i)
+      if (lst[i] == fld)
+         return lst[i - 1] = val;
+}
+
 /*** Form submit ***/
 function doPost(form) {
    for (var i = 0; ; ++i) {
@@ -212,7 +220,7 @@ function post(form, auto, exe, file) {
                else {
                   if (fld.type == "checkbox") {
                      fld.checked = val != "";
-                     document.getElementsByName(fld.name)[0].value = "";
+                     setCheck(fld, "");
                   }
                   else if (fld.type == "select-one") {
                      for (j = 0; j < fld.options.length; ++j) {
@@ -243,7 +251,7 @@ function post(form, auto, exe, file) {
                      fld.disabled = true;
                      InBtn = 0;  // 'onblur' on won't come when disabled
                      if (fld.type == "checkbox"  &&  fld.checked)
-                        document.getElementsByName(fld.name)[0].value = "T";
+                        setCheck(fld, "T");
                      ++i;
                   }
                   if (fld.pilDisable)
